package ru.dbotthepony.mc.otm.registry

import net.minecraft.core.Registry
import net.minecraft.resources.ResourceKey
import net.minecraft.resources.ResourceLocation
import net.minecraftforge.registries.ForgeRegistry
import net.minecraftforge.registries.IForgeRegistry
import net.minecraftforge.registries.NewRegistryEvent
import net.minecraftforge.registries.RegistryBuilder
import ru.dbotthepony.mc.otm.OverdriveThatMatters
import java.util.function.Supplier
import kotlin.properties.ReadOnlyProperty
import kotlin.reflect.KProperty

class RegistryDelegate<T>(key: String) : ReadOnlyProperty<Any, ForgeRegistry<T>>, Supplier<ForgeRegistry<T>> {
	private var value: Supplier<IForgeRegistry<T>?>? = null

	val location = ResourceLocation(OverdriveThatMatters.MOD_ID, key)
	val key: ResourceKey<Registry<T>> = ResourceKey.createRegistryKey(location)

	override fun get(): ForgeRegistry<T> {
		val supp = value ?: throw IllegalStateException("Tried to access uninitialized registry $location")
		return supp.get() as ForgeRegistry<T>? ?: throw IllegalStateException("Accessing registry $location too early")
	}

	override fun getValue(thisRef: Any, property: KProperty<*>): ForgeRegistry<T> {
		return get()
	}

	fun build(event: NewRegistryEvent) {
		if (value != null) {
			throw IllegalStateException("Already built registry $location!")
		}

		value = RegistryBuilder<T>().let {
			it.setName(location)
			// it.type = AndroidFeatureType::class.java
			event.create(it)
		}
	}
}
